/*
 * HtmlReader.java
 * Version 2.2
 * 2001/10/17
 * Written by Victor Liu See-le <victor@n-gon.com>
 * Copyright GPL
 */

package TheSlowArrowofBeauty;

import java.io.*;
import java.net.*;
import java.util.*;

/*
 * class HtmlReader    
 *
 * Reads text from an HTTP stream. 
 */
class HtmlReader {

    static final int INITIAL_READ_CAPACITY = 32768;
    
    private Hashtable specialChars = null;
    private Vector ignoreFileExtensions = null;
    private String appletSecurityEvader = null;
    
    HtmlReader() {
        specialChars = new Hashtable(31);
        specialChars.put("&nbsp;", new Character(' '));
        specialChars.put("&quot;", new Character('\"'));
        specialChars.put("&lt;", new Character('<'));
        specialChars.put("&gt;", new Character('>'));

        ignoreFileExtensions = new Vector(40);
        ignoreFileExtensions.addElement(".bin");
        ignoreFileExtensions.addElement(".lha");
        ignoreFileExtensions.addElement(".lzh");
        ignoreFileExtensions.addElement(".class");
        ignoreFileExtensions.addElement(".ai");
        ignoreFileExtensions.addElement(".eps");
        ignoreFileExtensions.addElement(".ps");
        ignoreFileExtensions.addElement(".rtf");
        ignoreFileExtensions.addElement(".Z");
        ignoreFileExtensions.addElement(".gtar");
        ignoreFileExtensions.addElement(".gz");
        ignoreFileExtensions.addElement(".zip");
        ignoreFileExtensions.addElement(".au");
        ignoreFileExtensions.addElement(".snd");
        ignoreFileExtensions.addElement(".mpga");
        ignoreFileExtensions.addElement(".mp2");
        ignoreFileExtensions.addElement(".aif");
        ignoreFileExtensions.addElement(".aiff");
        ignoreFileExtensions.addElement(".aifc");
        ignoreFileExtensions.addElement(".ram");
        ignoreFileExtensions.addElement(".rpm");
        ignoreFileExtensions.addElement(".ra");
        ignoreFileExtensions.addElement(".wav");
        ignoreFileExtensions.addElement(".gif");
        ignoreFileExtensions.addElement(".jpeg");
        ignoreFileExtensions.addElement(".jpg");
        ignoreFileExtensions.addElement(".jpe");
        ignoreFileExtensions.addElement(".png");
        ignoreFileExtensions.addElement(".tiff");
        ignoreFileExtensions.addElement(".tif");
        ignoreFileExtensions.addElement(".xbm");
        ignoreFileExtensions.addElement(".mpeg");
        ignoreFileExtensions.addElement(".mpg");
        ignoreFileExtensions.addElement(".mpe");
        ignoreFileExtensions.addElement(".qt");
        ignoreFileExtensions.addElement(".mov");
        ignoreFileExtensions.addElement(".avi");
        ignoreFileExtensions.addElement(".movie");
    }
    
    public String read(URL u) throws IOException {
        return read(u, -1);
    }

    public String read(URL u, int max) throws IOException {
        if (u == null) {
            throw new IOException("URL is null in htmlReader.read");
        }
        String s = "";
        int httpResponseCode;
        
        if (appletSecurityEvader != null) {
            // Create a new URL by prepending the appletSecurityEvader script.
            u = new URL(appletSecurityEvader + u.toString());
        }
        
        if (Debug.DEBUGGING_ON) {
            System.out.println("URL=" + u.toString());
        }
        
        URLConnection conn = u.openConnection();
        
        HttpURLConnection httpConn = null;
        try {
            if (conn instanceof HttpURLConnection) {
                httpConn = (HttpURLConnection) conn;
            }
        } catch (Exception e) {}
        
        conn.setUseCaches(false);
        conn.setRequestProperty("User-Agent", "TheSlowArrowofBeauty 2.2");
        conn.connect();
        
        if (Debug.DEBUGGING_ON) {
            for (int i=0; ; i++) {
                String value = conn.getHeaderField(i);
                if (value == null) 
                    break;
                String key = conn.getHeaderFieldKey(i);
                if (key == null)
                    System.out.println(value);
                else 
                    System.out.println(key + ": " + value);
            }
        }
        
        try {
            String contentType = conn.getContentType();
            if (contentType != null && !contentType.startsWith("text")) {
                throw new IOException("URL is not text");
            }
        } catch (NullPointerException npe) {
            if (Debug.DEBUGGING_ON) {
                npe.printStackTrace();
            }
            throw new IOException("Exception generated at getContentType()");
        }
        
        if (httpConn != null) {
            httpResponseCode = httpConn.getResponseCode();
        } else {
            httpResponseCode = 200;
        }

        if (httpResponseCode < 300) {
            int maxRead = conn.getContentLength();
            if (maxRead == -1) {
                maxRead = max;
            }
            InputStream in = conn.getInputStream();
            s = readStream(in, maxRead);
            in.close();
        } else {
            if (httpConn != null) {
                httpConn.disconnect();
                httpConn = null;
            }
            throw new IOException("HTTP Response code = " + httpResponseCode);
        }
        if (httpConn != null) {
            httpConn.disconnect();
        }

        return s;
    }
    
    public String readStreamLoTech(InputStream in, int max) 
            throws IOException {

        if (max == -1) {
            byte[] buf = ResourceManager.toByteArray(in);
            return (new String(buf));
        } else {
            byte[] buf = new byte[max];
            in.read(buf, 0, max);
            return (new String(buf));
        }
    }

    public String readStream(InputStream in, int max) 
            throws IOException {
            
        int t;
        StringBuffer sbuf = new StringBuffer();
        BufferedInputStream bin = 
                new BufferedInputStream(in, INITIAL_READ_CAPACITY);

        // Wait 10 seconds for input
        // Further, only allow 10 consecutive zero-read failures
        int consecutiveFailures = 0;
        for (t = 0; t < 10000; t += 50) {
            int nAvail = bin.available();
            if (Debug.DEBUGGING_ON) {
                System.out.println("readStream: n (available) =" + nAvail);
            }
            // nAvail = (nAvail == 0) ? 1 : nAvail;
            byte[] buf = new byte[nAvail];
            int nRead = bin.read(buf);
            if (Debug.DEBUGGING_ON) {
                System.out.println("readStream: n (read) =" + nRead);
            }
            if (nRead == -1) {
                break;
            }
            // Bug in Java Plug-in 1.3.0_01 for Linux:
            // read() does not return -1 to signal EOF
            // Use workaround for now and comment original. 
            // However, may end input stream prematurely if
            // there are no bytes available even though 
            // stream is not finished.
            if (nRead > 0) {
                sbuf.append(new String(buf));
                consecutiveFailures = 0;
            } else {
                consecutiveFailures++;
            }
            if (consecutiveFailures > 10) {
                break;
            }
            if ((max > -1) && (sbuf.length() >= max)) {
                break;
            }
            try { 
                Thread.sleep(50); 
            } catch (Exception e) {} 
        }
        if ((consecutiveFailures > 10) || (t == 10000)) {
            System.out.println("timeout waiting for HTTP stream");
        }
        bin.close();
        return (new String(sbuf));
    }

    public String getTagAttribute(String tag, String attribute) {
        StringTokenizer st = new StringTokenizer(tag);
        int n = st.countTokens();
        String token;
        for (int i = 0; i < n; i++) {
            token = st.nextToken();
            int p = token.indexOf('=');
            if (p == -1) {
                continue;
            }
            String thisAttrib = (token.substring(0, p)).trim();
            if (thisAttrib.equalsIgnoreCase(attribute)) { 
		String value = (token.substring(p + 1)).trim();
		if (value.startsWith("\"") || (value.startsWith("'"))) {
		    value = value.substring(1);
		}
		if (value.endsWith("\"") || (value.endsWith("'"))) {
		    value = value.substring(0, value.length() - 1);
		}
                return value;
            }
        }
        return "";
    }

    public String replaceSpecialChars(String s) {
        int p1 = 0; 
        int p2 = 0; 
        int lastPos = 0;
        StringBuffer sbuf = new StringBuffer(s.length());
        String code;

        for (;;) {
            p1 = s.indexOf('&', lastPos);
            if (p1 == -1) {
                sbuf.append(s.substring(lastPos));
                break;
            }
            p2 = s.indexOf(';', p1);
            if (p2 == -1){
                sbuf.append(s.substring(lastPos));
                break;
            }
            sbuf.append(s.substring(lastPos, p1));
            code = s.substring(p1, p2 + 1);
            if (specialChars.containsKey(code)) {
                sbuf.append(((Character) specialChars.get(code)).charValue());
            }
            lastPos = p2 + 1;
        }
        return new String(sbuf);
    }

    public boolean validLink(String link) {
        int pos = link.lastIndexOf('.');
        if (pos == -1) {
            return true;
        }
        String ext = link.substring(pos);
        return (!ignoreFileExtensions.contains(ext));
    }

    public static URL makeFullURL(String host, 
                                  int port, 
                                  String file, 
                                  String rel) {
        try {
            if (rel.startsWith("http://")) {
                return new URL(rel);
            }
            if (rel.startsWith("/")) {
                return new URL("http", host, port, rel);
            }
            int lastSlash = file.lastIndexOf('/');
            String path = file.substring(0, lastSlash + 1);
            while (rel.startsWith("../")) {
                lastSlash = path.lastIndexOf('/', path.length() - 2);
                path = path.substring(0, lastSlash + 1);
                rel = rel.substring(3);
            }
            while (rel.startsWith("./")) {
                rel = rel.substring(2);
            }
            return new URL("http", host, port, path + rel);
        } catch (MalformedURLException e) {
            if (Debug.DEBUGGING_ON) {
                System.err.println(e);
            }
            return null;
        }
    }

    public static URL makeFullURL(String ref, String rel) {
        try {
            URL u = new URL(ref);
            return makeFullURL(u.getHost(), u.getPort(), u.getFile(), rel);
        } catch (MalformedURLException e) {
            System.err.println(e);
            return null;
        }
    }

    public void setAppletSecurityEvader(String s) {
        appletSecurityEvader = s;
    }
}
