/*
 * ResourceManager.java    
 * Version 2.2
 * 2001/10/17
 * Written by Victor Liu See-le <victor@n-gon.com>
 * Copyright GPL
 */

package TheSlowArrowofBeauty;

import java.io.*;
import java.util.Vector;
import java.awt.*;

/**
 * class ResourceManager
 *
 * Manages resource retrieval.
 */
class ResourceManager {

    public Vector readLinesFromFile(String fileName, int initialCapacity) 
            throws IOException {
        Vector v = new Vector(initialCapacity);
        BufferedReader reader;
        InputStream in = getClass().getResourceAsStream(fileName);
        reader = new BufferedReader(new InputStreamReader(in));
        String s = null;
        while ((s = reader.readLine()) != null) {
            v.addElement(s.trim());
        }
        reader.close();
        v.trimToSize();
        return v;
    }

    public Image getImage(String fileName) throws IOException {
        InputStream in = getClass().getResourceAsStream(fileName);
        if (in == null) {
            throw new IOException("Image " + fileName + " not found");
        }
        final byte[] buffer = toByteArray(in);
        in.close();
        BufferedInputStream bin = new BufferedInputStream(in);
        //        int n = bin.available();
        //        byte[] buffer = new byte[n];
        //        bin.read(buffer);
        return Toolkit.getDefaultToolkit().createImage(buffer);
    }
    
    // This method found on Java Developer forum
    public static byte[] toByteArray(final InputStream input) 
            throws IOException {
        int status = 0;
        final int blockSize = 5000;
        int totalBytesRead = 0;
        int blockCount = 1;
        byte[] dynamicBuffer = new byte[blockSize*blockCount];
        final byte[] buffer = new byte[blockSize];
        
        boolean endOfStream = false;
        while (!endOfStream) {
            int bytesRead = 0;
            if (input.available() != 0) {
                // data is waiting so read as
                // much as is available
                status = input.read(buffer);
                endOfStream = (status == -1);
                if (!endOfStream)
                    bytesRead = status;
            } else {
                // no data waiting so use the
                // one character read to block until
                // data is available or the end of the input stream is reached
                status = input.read();
                endOfStream = (status == -1);
                buffer[0] = (byte)status;
                if (!endOfStream) {
                    bytesRead = 1;
                }
            }
            
            if (!endOfStream) {
                if (totalBytesRead + bytesRead > blockSize*blockCount) {
                    // expand the size of the buffer
                    blockCount++;
                    final byte[] newBuffer = new byte[blockSize*blockCount];
                    System.arraycopy(dynamicBuffer, 0,
                                     newBuffer, 0, totalBytesRead);
                    dynamicBuffer = newBuffer;
                }
                System.arraycopy(buffer, 0,
                                 dynamicBuffer, totalBytesRead, bytesRead);
                totalBytesRead += bytesRead;
            }
        }
        
        // make a copy of the array of the exact length
        final byte[] result = new byte[totalBytesRead];
        if (totalBytesRead != 0) {
            System.arraycopy(dynamicBuffer, 0, result, 0, totalBytesRead);
        }
        
        return result;
    }
}

