/*
 * SearchEngineAnalyzer.java
 * Version 2.2
 * 2001/10/17
 * Written by Victor Liu See-le <victor@n-gon.com>
 * Copyright GPL
 */
 
package TheSlowArrowofBeauty;

import java.net.*;
import java.io.*;
import java.util.*;

/*
 * class SearchEngineAnalyzer    
 *
 * Logic for finding delimiters of links on a search results page.
 */
class SearchEngineAnalyzer {

    public static final String QUERY = "Plato";

    private String domain;
    private HtmlReader htmlReader;
    private Vector links;
    private Hashtable preCandidates;
    private Hashtable postCandidates;
    private String html;
    private Link curLink;
    //private SearchEngineInterface parent;

    SearchEngineAnalyzer(String domain) {
        this.domain = domain;
        htmlReader = new HtmlReader();
        links = new Vector(128);
        preCandidates = new Hashtable(31);
        postCandidates = new Hashtable(31);
    }

    public SearchEngineResultHint lookForSearchResultHints(String cgiCall) {

        try {
            URL u = new URL(cgiCall + QUERY);
            html = htmlReader.read(u);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }

        int p1;
        int p2 = 0;
        char c;
        String tag;
        boolean inLink = false;
        while ((p1 = html.indexOf('<', p2)) != -1) {
            p2 = html.indexOf('>', p1 + 1);
            if (p2 == -1) {
                break;
            }
            // What kind of tag?
            String s = html.substring(p1 + 1, p2);
            StringTokenizer st = new StringTokenizer(s);
            tag = st.nextToken();
            if (tag.equalsIgnoreCase("A")) {

                // If already in link, then we didn't find </A> tag
                if (inLink) {
                    curLink.endPos = p1 - 1;
                    curLink.text = "";
                }

                String href = htmlReader.getTagAttribute(s, "HREF");
                Link link = new Link(p1 + 1 + s.indexOf(href), href);
                links.addElement(link);
                curLink = link;
                inLink = true;
                
                // Look for closing </A> tag
                int pSave = p2;
                while ((p1 = html.indexOf("</", p2)) != -1) {
                    p2 = html.indexOf('>', p1 + 1);
                    if (p2 == -1) {
                        p2 = html.length();
                    }
                    // What kind of tag?
                    s = html.substring(p1 + 2, p2);
                    st = new StringTokenizer(s);
                    tag = st.nextToken();
                    if (tag.equalsIgnoreCase("A")) {
                        s = html.substring(pSave + 1, p1);
                        curLink.endPos = p1;
                        curLink.text = s;
                        inLink = false;

                        break;
                    }
                }
            } 
        }

        lookForPreCandidates("");
        lookForPostCandidates("");

        // Store prefix and suffix candidates with links
        int nLinks = links.size();
        Link link;
        int nPreCandidates = preCandidates.size();
        int nPostCandidates = postCandidates.size();
        for (int i = 0; i < nLinks; i++) {
            link = (Link) links.elementAt(i);
            String candidate;
            int len;
            int pos;
            Enumeration candidates = preCandidates.keys();
            while (candidates.hasMoreElements()) {
                candidate = (String) candidates.nextElement();
                len = candidate.length();
                pos = link.beginPos;
                if (html.regionMatches(pos - len, candidate, 0, len)) {
                    link.hint.prefix = candidate;
                    break;
                }
            }
            candidates = postCandidates.keys();
            while (candidates.hasMoreElements()) {
                candidate = (String) candidates.nextElement();
                len = candidate.length();
                pos = link.endPos;
                if (html.regionMatches(pos, candidate, 0, len)) {
                    link.hint.suffix = candidate;
                    break;
                }
            }
        }  


        // Finally, try to arrive at hint pair.
        Enumeration enum = links.elements();
        Hashtable hintPairs = new Hashtable(31);
        int maxOccurencesSoFar = 0;
        String leaderPrefix = "";
        String leaderSuffix = "";
        while (enum.hasMoreElements()) {
            link = (Link) enum.nextElement();

            // Skip those with no hint candidates
            if (link.hint.prefix == null && link.hint.suffix == null) {
                continue;
            }

            // Skip those whose link contains this domain name
            if (link.href.indexOf(domain) != -1) {
                continue;
            }

            // Now search through remaining links for those whose 
            // text contains the query string. Count occurences of
            // prefix/suffix pairs, and choose the pair with the
            // highest count.
            if ((link.text.toLowerCase()).indexOf(QUERY.toLowerCase()) != -1) {
                String s = link.hint.prefix + QUERY + link.hint.suffix;
                if (hintPairs.containsKey(s)) {
                    int n = ((Integer) hintPairs.get(s)).intValue();
                    hintPairs.put(s, new Integer(n + 1));
                    if (n + 1 > maxOccurencesSoFar) {
                        maxOccurencesSoFar = n + 1;
                        leaderPrefix = link.hint.prefix;
                        leaderSuffix = link.hint.suffix; 
                    }
                } else {
                    hintPairs.put(s, new Integer(1));
                }

            }
        }
        return new SearchEngineResultHint(leaderPrefix, leaderSuffix);
    }
    
    private void lookForPreCandidates(String soFar) {

        int nLinks = links.size();
        //Hashtable nextCharCandidates = new Hashtable(nLinks);
        Hashtable nextCharCandidates = new Hashtable(31);
        int pos;
        for (int i = 0; i < nLinks; i++) {
            pos = ((Link) links.elementAt(i)).beginPos;
            int len = soFar.length();
        if (pos < len) {
                continue;
            }
            if (len > 0) {
                // Check if matches soFar
                if (!html.regionMatches(pos - len, soFar, 0, len)) {
                    continue;
                }
            }
            char c = html.charAt(pos - (len + 1));
            Character C = new Character(c);
            if (nextCharCandidates.containsKey(C)) {
                int n = ((Integer) nextCharCandidates.get(C)).intValue();
                nextCharCandidates.put(C, new Integer(n + 1));
            } else {
                nextCharCandidates.put(C, new Integer(1));
            }
        }

        // Hold onto the ones which have greater than 10 occurrences
        Enumeration keys = nextCharCandidates.keys();
        Character C;
        char c;
        Integer N;
        int n;
        while (keys.hasMoreElements()) {
            C = (Character) keys.nextElement();
            c = C.charValue();
            N = (Integer) nextCharCandidates.get(C);
            n = N.intValue();
            if (n >= 10) {
                String s = new String(c + soFar);
                preCandidates.remove(soFar);
                preCandidates.put(s, N);
                lookForPreCandidates(s);
            }
        }
    }

    private void lookForPostCandidates(String soFar) {

        int nLinks = links.size();
        //Hashtable nextCharCandidates = new Hashtable(nLinks);
        Hashtable nextCharCandidates = new Hashtable(31);
        int pos;
        for (int i = 0; i < nLinks; i++) {
            pos = ((Link) links.elementAt(i)).endPos;
            int len = soFar.length();
            if (pos + len >= html.length()) {
                continue;
            }
            if (len > 0) {
                // Check if matches soFar
                if (!html.regionMatches(pos, soFar, 0, len)) {
                    continue;
                }
            }
            char c = html.charAt(pos + len);
            Character C = new Character(c);
            if (nextCharCandidates.containsKey(C)) {
                int n = ((Integer) nextCharCandidates.get(C)).intValue();
                nextCharCandidates.put(C, new Integer(n + 1));
            } else {
                nextCharCandidates.put(C, new Integer(1));
            }
        }

        // Hold onto the ones which have greater than 10 occurrences
        Enumeration keys = nextCharCandidates.keys();
        Character C;
        char c;
        Integer N;
        int n;
        while (keys.hasMoreElements()) {
            C = (Character) keys.nextElement();
            c = C.charValue();
            N = (Integer) nextCharCandidates.get(C);
            n = N.intValue();
            if (n >= 10) {
                String s = new String(soFar + c);
                postCandidates.remove(soFar);
                postCandidates.put(s, N);
                lookForPostCandidates(s);
            }
        }
    }

    public void setAppletSecurityEvader(String s) {
        htmlReader.setAppletSecurityEvader(s);
    }
    
    private class Link {
        int beginPos;
        int endPos;
        String href;
        String text;
        SearchEngineResultHint hint;

        Link() {
            super();
            hint = new SearchEngineResultHint();
        }

        Link(int beginPos, String href) {
            this();
            this.beginPos = beginPos;
            this.href = href;
        }
    }
}
