/*
 * Slider.java
 * Version 2.2
 * 2001/10/17
 * Written by Victor Liu See-le <victor@n-gon.com>
 * Copyright GPL
 */

package TheSlowArrowofBeauty;

import java.awt.*;
import java.lang.Math;
    
/*
 * class Slider 
 * 
 * A canvas which slides when moused-over.
 */
class Slider extends Canvas implements Runnable {

    protected static final int WAITING         = 0;
    protected static final int SLIDING_OUT     = 1;
    protected static final int SLIDING_IN      = 2;

    protected Point origin;
    protected Point slideExtent;
    protected Point slideInc;
    protected int slideSteps;
    protected int slideDelay;
    protected Point slidePos;
    protected volatile Thread anim = null;
    protected int state;

    Slider(Point origin, Point slideExtent, int slideSteps, int slideDelay) {
        super();
        this.origin = origin;
        this.slideExtent = slideExtent;
        this.slideSteps = slideSteps;
        this.slideDelay = slideDelay;
        slideInc = new Point(slideExtent.x / slideSteps, 
                             slideExtent.y / slideSteps);
        slidePos = new Point(0, 0);
        setLocation(origin);
        state = WAITING;
        start();
    }

    public void start() {
        if (anim == null) {
            anim = new Thread(this);
        }
        anim.setDaemon(true);
        anim.setPriority(Thread.MAX_PRIORITY);
        anim.start();
    }

    public synchronized void stop() { 
        if (anim != null) {
            Thread moribund = anim;
            anim = null;
            moribund.interrupt(); 
        }
    }
    
    public synchronized void slideOut() {
        requestFocus();
        state = SLIDING_OUT;
        notify();
    }

    public synchronized void slideIn() {
        transferFocus();
        state = SLIDING_IN;
        notify();
    }

    public void run() {
        Thread currentThread = Thread.currentThread();
        while (currentThread == anim) {
            switch (state) {
            case SLIDING_OUT:
                if ((Math.abs(slidePos.x) < Math.abs(slideExtent.x)) ||
                        (Math.abs(slidePos.y) < Math.abs(slideExtent.y))) {
                        
                    slidePos.translate(slideInc.x, slideInc.y);
                    if (Math.abs(slidePos.x) > Math.abs(slideExtent.x)) {
                        slidePos.x = slideExtent.x;
                    }
                    if (Math.abs(slidePos.y) > Math.abs(slideExtent.y)) {
                        slidePos.y = slideExtent.y;
                    }
                    if (slidePos.x == slideExtent.x && 
                            slidePos.y == slideExtent.y) {
                        state = WAITING;
                    }
                    setLocation(origin.x + slidePos.x, origin.y + slidePos.y);
                    //repaint();
                    try {
                        anim.sleep(slideDelay);
                    } catch (InterruptedException e) {}
                }
                break;
                    
            case SLIDING_IN:
                if (slidePos.x != 0 || slidePos.y != 0) {
                        
                    slidePos.translate(-slideInc.x, -slideInc.y);
                    if ((slideInc.x > 0 && slidePos.x < 0) || 
                            (slideInc.x < 0 && slidePos.x > 0)) {
                        slidePos.x = 0;
                    }
                    if ((slideInc.y > 0 && slidePos.y < 0) || 
                            (slideInc.y < 0 && slidePos.y > 0)) {
                        slidePos.y = 0;
                    }
                    if (slidePos.x == 0 && slidePos.y == 0) {
                        state = WAITING;
                    }
                    setLocation(origin.x + slidePos.x, origin.y + slidePos.y);
                    //repaint();
                    try {
                        anim.sleep(slideDelay);
                    } catch (InterruptedException e) {}
                }
                break;
                    
            case WAITING:
                try {
                    synchronized(this) {
                        while (state == WAITING) {
                            wait();
                        }
                    } 
                } catch (InterruptedException e) {}
                break;
            }
        }
    }
}


