/*
 * SlowArrow.java
 * Version 2.2
 * 2001/10/17
 * Written by Victor Liu See-le <victor@n-gon.com>
 * Copyright GPL
 */

package TheSlowArrowofBeauty;

import java.applet.Applet;
import java.io.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import netscape.javascript.*;

/**
 * class SlowArrow
 *
 * Root class runs as both applet and application.
 */
public class SlowArrow extends Applet implements KeyListener {

    SlowArrowConfig defaults;
    SlowArrowConfig conf;
    SlowArrowDisplay d[];
    SlowArrowProgressIndicator p[];
    SlowArrowHistory history;
    SlowArrowKeywordsList keywordsList;
    SlowArrowProgressIndicatorTimer piTimer;
    SlowArrowKiller killer;
    SlowArrowHelper helper;
    SlowArrowHelpPanel helpPanel = null;
    SearchEngineInterface searchEngines;
    ResourceManager rcm;
    int maxKeywords;
    int numKeywords2Match;
    Vector stopwords;
    boolean isRunningAsApplet = false;
    KeyListener currentKeyListener = null;
    private int whichScheme[];
    private Dimension screenDim = null;
    private String keywordsFile;
    
    public static void main (String args[]) {
        Frame f = new Frame("The Slow Arrow of Beauty");
        Window w = new Window(f);
        w.setLayout(null);
        SlowArrow sa = new SlowArrow();
        sa.isRunningAsApplet = false;
        sa.initSlowArrow();

        int numArgs = args.length;
        if (numArgs > 0) {

            // Look for an external configuration file.
            // If it exists, apply configs listed therein
            // before applying any command-line configs.
            for (int i = 0; i < numArgs; i++) {
                if (args[i].startsWith("configFile")) {
                    int sep = args[i].indexOf('=');
                    if (sep > -1) {
                        String fileName = args[i].substring(sep + 1);
                        try {
                            Vector v = sa.rcm.readLinesFromFile(fileName, 20);
                            int n = v.size();
                            for (int j = 0; j < n; j++) {
                                sa.conf.applyConfig((String) v.elementAt(j));
                            }
                        } catch (IOException e) {
                            if (Debug.DEBUGGING_ON) {
                                System.err.println(
                                        "Error reading from config file " +
                                        fileName);
                                e.printStackTrace();
                            }
                        }
                    }
                }
            }

            // Finally apply any command-line arguments.
            String param;
            String value;
            for (int i = 0; i < numArgs; i++) {
                sa.conf.applyConfig(args[i]);
            }
        }

        sa.initSearchEngines();
        w.add(sa);
        sa.screenDim = f.getToolkit().getScreenSize();
        sa.initComponents();
        w.setBounds(0, 0, sa.screenDim.width, sa.screenDim.height);
        w.setVisible(true);
        w.show();
        f.addKeyListener(sa);
        f.show(); // Necessary to get key events (at least on linux)
        sa.start();
    }

    public void init() {
        isRunningAsApplet = true;
        initSlowArrow();

        // Get parameters. Go through list of defaults
        // and check if there's an overriding parameter in the HTML.
        Enumeration keys = defaults.propertyNames();
        String key;
        String val;
        while (keys.hasMoreElements()) {
            key = (String) keys.nextElement();
            val = getParameter(key);
            if (val != null) {
                conf.put(key, val);
            }
        }

        initSearchEngines();
        initComponents();
        addKeyListener(this);
    }

    private void initSlowArrow() {

        // First check for Java version
        String javaVersion = System.getProperty("java.version");
        StringTokenizer st = new StringTokenizer(javaVersion, ".");
        int majorVersion = Integer.parseInt(st.nextToken());
        int minorVersion = Integer.parseInt(st.nextToken());
        if (!(majorVersion >= 1 && minorVersion >= 1)) {
            String err = new String("The version of Java you are running is "
                                    + javaVersion +
                                    ": Version 1.1 is required.");
            if (isRunningAsApplet == true) {
                showStatus(err);
            }
            System.err.println(err);
            destroy();
        }

        setLayout(null);
        setBackground(Color.black);
        setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
        defaults = new SlowArrowConfig(SlowArrowConfig.DEFAULT);
        conf = new SlowArrowConfig(defaults);
        rcm = new ResourceManager();
    }

    private void initSearchEngines() {

        searchEngines = new SearchEngineInterface();
        
        String appletSecurityEvader = 
                conf.getStringProperty("appletSecurityEvader");
        if (appletSecurityEvader.length() > 0) {
            searchEngines.setAppletSecurityEvader(appletSecurityEvader);
        }
        
        try {
            searchEngines.init();
        } catch (IOException e) {
            System.err.print("IOException while initializing ");
            System.err.println("SearchEngineInterface");
            e.printStackTrace();
            kill();
        }
    }
    
    private void initComponents() {

        int killerX = conf.getIntegerProperty("killerX");
        int killerY = conf.getIntegerProperty("killerY");
        int killerW = conf.getIntegerProperty("killerWidth");
        int killerH = conf.getIntegerProperty("killerHeight");
        int killerSlideExtentX = conf.getIntegerProperty("killerSlideExtentX");
        int killerSlideExtentY = conf.getIntegerProperty("killerSlideExtentY");
        int killerSlideSteps = conf.getIntegerProperty("killerSlideSteps");
        int killerSlideDelay = conf.getIntegerProperty("killerSlideDelay");
        Point killerOrigin = new Point(killerX, killerY);
        Point killerSlideExtent = new Point(killerSlideExtentX,
                                            killerSlideExtentY);
        killer = new SlowArrowKiller(killerOrigin,
                                     killerSlideExtent,
                                     killerSlideSteps,
                                     killerSlideDelay,
                                     this);
        killer.setBounds(killerX, killerY, killerW, killerH);
        add(killer);

        int helperX = conf.getIntegerProperty("helperX");
        int helperY = conf.getIntegerProperty("helperY");
        int helperW = conf.getIntegerProperty("helperWidth");
        int helperH = conf.getIntegerProperty("helperHeight");
        int helperSlideExtentX = conf.getIntegerProperty("helperSlideExtentX");
        int helperSlideExtentY = conf.getIntegerProperty("helperSlideExtentY");
        int helperSlideSteps = conf.getIntegerProperty("helperSlideSteps");
        int helperSlideDelay = conf.getIntegerProperty("helperSlideDelay");
        Point helperOrigin = new Point(helperX, helperY);
        Point helperSlideExtent = new Point(helperSlideExtentX,
                                            helperSlideExtentY);
        helper = new SlowArrowHelper(helperOrigin,
                                     helperSlideExtent,
                                     helperSlideSteps,
                                     helperSlideDelay,
                                     this);
        helper.setBounds(helperX, helperY, helperW, helperH);
        add(helper);

        int left = conf.getIntegerProperty("left");
        int top = conf.getIntegerProperty("right");
        int width = conf.getIntegerProperty("width");
        int height = conf.getIntegerProperty("height");
        int nSchemes = conf.getIntegerProperty("numDisplayColorSchemes");
        int nDisplays = conf.getIntegerProperty("numDisplays");

        // Center if running as stand-alone app.
        if (screenDim != null) {
            left = (screenDim.width - width) / 2;
            top = (screenDim.height - height) / 2;
        }

        setBounds(left, top, width, height);

        int maxHistory = conf.getIntegerProperty("maxHistory");
        history = new SlowArrowHistory(maxHistory);

        int[] whichScheme = new int[nDisplays];
        for (int i = 0; i < nDisplays; i++) {
            boolean fail;
            int j;
            do {
                fail = false;
                whichScheme[i] = (int) (Math.random() * nSchemes);
                for (j = 0; j < i; j++) {
                    if (whichScheme[i] == whichScheme[j]) {
                        fail = true;
                        break;
                    }
                }
            } while (fail == true);
        }

        d = new SlowArrowDisplay[nDisplays];
        p = new SlowArrowProgressIndicator[nDisplays];
        int dTop = conf.getIntegerProperty("displayTop");
        int dWidth = conf.getIntegerProperty("displayWidth");
        int dHeight = conf.getIntegerProperty("displayHeight");
        int dSpc = conf.getIntegerProperty("displaySpacing");
        int piBoxW = conf.getIntegerProperty("progressIndicatorBoxWidth");
        int piBoxH = conf.getIntegerProperty("progressIndicatorBoxHeight");
        int piMargin =
                conf.getIntegerProperty("progressIndicatorBottomMargin");
        int d0Left = (getSize().width -
                      (nDisplays * dWidth) -
                      ((nDisplays - 1) * dSpc)) / 2;
        for (int i = 0; i < nDisplays; i++) {
            int dLeft = d0Left + i * (dWidth + dSpc);
            p[i] = new SlowArrowProgressIndicator(this);
            p[i].setColorScheme(whichScheme[i]);
            p[i].setBounds(dLeft,
                           dHeight - piBoxH - piMargin,
                           dWidth,
                           piBoxH);
            p[i].setVisible(false);
            d[i] = new SlowArrowDisplay(this, p[i]);
            d[i].setBounds(dLeft, dTop, dWidth, dHeight);
            d[i].setColorScheme(whichScheme[i]);
            d[i].setVisible(false);
            add(p[i]);
            add(d[i]);
        }

        setVisible(true);

        piTimer = new SlowArrowProgressIndicatorTimer(this);

        String stopwordsFile = conf.getProperty("stopwordsFile");
        try {
            stopwords = rcm.readLinesFromFile(stopwordsFile, 200);
        } catch (IOException e) {
            System.err.println("Error reading stopwords from file " +
                               stopwordsFile);
            e.printStackTrace();
        }

        maxKeywords = conf.getIntegerProperty("maxKeywords");
        keywordsFile = conf.getProperty("keywordsFile");
        keywordsList = new SlowArrowKeywordsList(this);
        try {
            Vector keywords = rcm.readLinesFromFile(keywordsFile, maxKeywords);
            int count = keywords.size();
            for (int i = count - 1; i >= 0; i--) {
                String s = (String) keywords.elementAt(i);
                StringTokenizer st = new StringTokenizer(s, "\t");
                String keyword = st.nextToken();
                int weight = Integer.parseInt(st.nextToken());
                keywordsList.addKeyword(keyword, weight);
            }
        } catch (IOException e) {
            System.err.println("Error reading keywords from file " +
                               keywordsFile);
            e.printStackTrace();
        }

        int klHeight = conf.getIntegerProperty("keywordsHeight");
        int klTopMargin = conf.getIntegerProperty("keywordsTopMargin");
        int klTop = dTop + dHeight + klTopMargin;
        keywordsList.setBounds(left, klTop, width, klHeight);
        add(keywordsList);

        numKeywords2Match = conf.getIntegerProperty("numKeywords2Match");
    }

    public void start() {
        helper.setVisible(true);
        helper.repaint();
        keywordsList.setVisible(true);
        keywordsList.repaint();
        playIntro(d[0].getBounds().x, 0);
        startDisplays();
    }

    private void playIntro(int x, int y) {
        SlowArrowScrollIntro scrollIntro = new SlowArrowScrollIntro(this);
        scrollIntro.move(x, y);
        add(scrollIntro);
        try {
            Thread.sleep(3000);
        } catch (InterruptedException e) {}

        scrollIntro.setVisible(true);
        scrollIntro.start();
        while (!scrollIntro.isDone()) {
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {}
        }

        int nDisplays = d.length;
        for (int i = 0; i < nDisplays; i++) {
            d[i].setVisible(true);
            p[i].setVisible(true);
            d[i].repaint();
            p[i].repaint();
        }
        scrollIntro.setVisible(false);
        remove(scrollIntro);
        scrollIntro = null;
    }

    private void startDisplays() {
        piTimer.start();
        int nDisplays = d.length;
        for (int i = 0; i < nDisplays; i++) {
            d[i].start();
        }
    }

    void showHelpPanel() {
        if (helpPanel == null) {
            helpPanel = new SlowArrowHelpPanel(this);
            add(helpPanel, 1);
        } else {
            helpPanel.setVisible(true);
        }
    }

    void hideHelpPanel() {
        if (helpPanel == null) {
            return;
        }
        helpPanel.setVisible(false);
    }

    void center(Component c) {
        Dimension saDim = getSize();
        Dimension cDim = c.getSize();
        int x = (saDim.width - cDim.width) / 2;
        int y = (saDim.height - cDim.height) / 2;
        c.setLocation(x, y);
    }

    public void keyTyped(KeyEvent e) {
        if (currentKeyListener != null) {
            currentKeyListener.keyTyped(e);
        }
    }

    public void keyPressed(KeyEvent e) {
        if (currentKeyListener != null) {
            currentKeyListener.keyPressed(e);
        }
    }

    public void keyReleased(KeyEvent e) {
        if (currentKeyListener != null) {
            currentKeyListener.keyReleased(e);
        }
    }

    public void stop() {
        super.stop();
        if (piTimer != null) {
            piTimer.stop();
        }
        if (d != null) {
            int n = d.length;
            for (int i = 0; i < n; i++) {
                d[i].stop();
            }
        }
    }
    
    public void kill() {
        if (!isRunningAsApplet) {
            try {
                FileWriter writer = new FileWriter(keywordsFile);
                BufferedWriter bwriter = new BufferedWriter(writer);
                int n = keywordsList.listSize();
                for (int i=0; i<n; i++) {
                    bwriter.write(keywordsList.getKeyword(i));
                    bwriter.write("\t");
                    bwriter.write(String.valueOf(keywordsList.getWeight(i)));
                    bwriter.write("\n");
                }
                bwriter.close();
                writer.close();
            } catch (IOException e) {}
            System.exit(0);
        } else {
            JSObject win = JSObject.getWindow(this);
            win.eval("top.close()");
        }
    }
}


