/*
 * SlowArrowHelper.java    
 * Version 2.2
 * 2001/10/17
 * Written by Victor Liu See-le <victor@n-gon.com>
 * Copyright GPL
 */

package TheSlowArrowofBeauty;

import java.awt.*;
import java.awt.event.*;

/*
 * class SlowArrowHelper 
 * 
 * Accesses Help screen.
 */
class SlowArrowHelper extends Slider implements MouseListener {

    private SlowArrow parent;
    private Image img = null;
    private int scrollPos;

    SlowArrowHelper(Point origin, 
                    Point slideExtent, 
                    int slideSteps, 
                    int slideDelay,
                    SlowArrow parent) {
        super(origin, slideExtent, slideSteps, slideDelay);
        this.parent = parent;
        addMouseListener(this);
    }

    private void initGraphics() {
        Dimension d = getSize();

        int tabW = parent.conf.getIntegerProperty("helperTabWidth");
        int fontSize = parent.conf.getIntegerProperty("fontSize");
        Color backC = parent.conf.getColorProperty("helperBackColor");
        Color textC = parent.conf.getColorProperty("helperTextColor");
        Color tabC = parent.conf.getColorProperty("helperTabColor");
        Color qC = parent.conf.getColorProperty("helperQMarkColor");
        Color outlineC = parent.conf.getColorProperty("helperTabOutlineColor");
        int tabX = d.width - tabW;
        
        img = createImage(d.width, d.height);
        Graphics gc = img.getGraphics();
        
        gc.setColor(backC);
        gc.fillRect(0, 0, tabX, d.height);
        gc.setColor(tabC);
        gc.fillRect(tabX, 0, tabW, d.height);
        
        gc.setColor(outlineC);
        gc.fillRect(d.width - 3, d.height - 3, 3, 3);
        gc.fillRect(d.width - 3, 0, 3, 3);
        
        int qX = tabX + (tabW - 10 - 3) / 2;
        int qY = (d.height - 9) / 2;
        gc.setColor(qC);
        gc.fillOval(qX + 2, qY, 7, 6);
        gc.fillOval(qX + 3, qY + 7, 3, 3);
        gc.fillRect(qX + 3, qY + 4, 2, 2);
        gc.setColor(tabC);
        gc.fillRect(qX + 1, qY + 2, 6, 2);
        
        Font font = new Font("Courier", Font.PLAIN, fontSize);
        FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(font);
        int asc = fm.getAscent();
        String s = "help";
        int sWidth = fm.stringWidth(s);
        gc.setFont(font);
        gc.setColor(textC);
        gc.drawString(s, tabX - sWidth - 5, (d.height - asc) / 2 + asc);
    }

    public void paint(Graphics g) {
        if (img == null) {
            initGraphics();
        }
        g.drawImage(img, 0, 0, this);
    }

    public void update(Graphics g) {
        paint(g);
    }

    public void mouseEntered(MouseEvent e) {
        e.consume();
        slideOut();
    }

    public void mouseExited(MouseEvent e) {
        e.consume();
        slideIn();
    }

    public void mousePressed(MouseEvent e) {}

    public void mouseReleased(MouseEvent e) {}

    public void mouseClicked(MouseEvent e) {
        e.consume();
        parent.showHelpPanel();
    }
}


