/*
 * SlowArrowKeywordsList.java    
 * Version 2.2
 * 2001/10/17
 * Written by Victor Liu See-le <victor@n-gon.com>
 * Copyright GPL
 */

package TheSlowArrowofBeauty;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/*
 * class SlowArrowKeywordsList 
 * 
 * Panel for keywords textbox and scrollers
 */
class SlowArrowKeywordsList extends Panel implements MouseListener, 
                                                     MouseMotionListener {

    final static int NORMAL         = 0;
    final static int LEFT_HILITE    = 1;
    final static int RIGHT_HILITE   = 2;

    SlowArrow parent;
    int max;
    int fontSize;
    Color backColor;
    Color textColor;
    Color hiliteColor;
    Color weightColor;
    private int width;
    private int height;
    SlowArrowKeywordsListTextbox textbox;
    SlowArrowKeywordsListScroller right,left;
    SlowArrowKeywordsListEditbox editbox;
    Log log;
    private Vector keywords;
    private boolean isDragging;
    private int dragX0;
    private int dragY0;
    private int mouseStrikeDistance;
    private int keywordsBaseWeight;
    private int textboxWidth;

    SlowArrowKeywordsList(SlowArrow parent) {
        this.parent = parent;
        this.max = parent.maxKeywords;
        keywords = new Vector(max);

        fontSize = 
                parent.conf.getIntegerProperty("fontSize");
        int numSchemes = 
                parent.conf.getIntegerProperty("keywordsNumColorSchemes");
        mouseStrikeDistance = 
                parent.conf.getIntegerProperty("mouseStrikeDistance");
        keywordsBaseWeight = 
                parent.conf.getIntegerProperty("keywordsBaseWeight");
        String logFile = 
                parent.conf.getStringProperty("keywordsLogFile");
        textboxWidth = 
                parent.conf.getIntegerProperty("keywordsTextboxWidth");

        int scheme = (int) (Math.random() * numSchemes);
        backColor = 
                parent.conf.getColorProperty("keywordsListBackColor" + 
                                             scheme);
        textColor =  
                parent.conf.getColorProperty("keywordsListTextColor" + 
                                             scheme);
        hiliteColor = 
                parent.conf.getColorProperty("keywordsListHiliteColor" + 
                                             scheme);
        weightColor = 
                parent.conf.getColorProperty("keywordsListWeightColor" + 
                                             scheme);

        setLayout(null);
        setBackground(Color.black);

        textbox = new SlowArrowKeywordsListTextbox(this);
        textbox.addMouseListener(this);
        textbox.addMouseMotionListener(this);
        add(textbox);

        right = new SlowArrowKeywordsListScroller(this);
        right.addMouseListener(this);
        add(right);
        
        left = new SlowArrowKeywordsListScroller(this);
        left.addMouseListener(this);
        add(left);

        editbox = new SlowArrowKeywordsListEditbox(this);
        add(editbox);

        if (parent.isRunningAsApplet == false) {
            log = new Log(logFile);
        }
        spill("Initializing");
    }

    synchronized boolean addKeyword(String s) {
        return addKeyword(s, keywordsBaseWeight);
    }

    synchronized boolean addKeyword(String s, int weight) {
        int i;

        // Check if stop word.
        if (parent.stopwords.contains(s.toLowerCase())) {
            return false;
        }

        int numKeywords = keywords.size();

        // Check if already there
        for (i = 0; i < numKeywords; i++) {
            if (s.equalsIgnoreCase(getKeyword(i))) {
                s = getKeyword(i);
                int wt = getWeight(i);
                weight += wt;
                keywords.removeElementAt(i);
                numKeywords--;
            }
        }

        Date now = new Date();
        spill("Adding keyword " + s + " with weight " + weight + 
              " [" + now + "]");

        // If at max load, remove last element.
        if (numKeywords == max) {
            spill("Dropping last item " + getKeyword(numKeywords - 1));
            keywords.removeElementAt(numKeywords - 1);
            numKeywords--;
        }

        // Find where to insert.
        for (i = 0; i < numKeywords; i++) {
            if (weight >= getWeight(i)) {
                break;
            }
        }
        keywords.insertElementAt(new SlowArrowKeyword(s, weight), i);
        numKeywords ++;
        textbox.scrollPos = i * textboxWidth;

        if (isShowing()) {
            int sigma = 0;
            for (i = 0; i < numKeywords; i++) {
                sigma += getWeight(i);
            }
            if (sigma > numKeywords * (keywordsBaseWeight + 1)) {
                for (i = 0; i < numKeywords; i++) {
                    ((SlowArrowKeyword) keywords.elementAt(i)).weight--;
                    if (getWeight(i) == 0) {
                        keywords.removeElementAt(i);
                    }
                }
            }

            textbox.keywordAdded(s);
        }

        return true;
    }

    synchronized void strikeKeyword() {
        int i = textbox.scrollPos / textboxWidth;
        spill("Striking keyword " + getKeyword(i));
        keywords.removeElementAt(i);
        textbox.keywordStriked();
    }

    synchronized String getKeyword(int i) {
        return ((SlowArrowKeyword) keywords.elementAt(i)).keyword;
    }

    synchronized int getWeight(int i) {
        return ((SlowArrowKeyword) keywords.elementAt(i)).weight;
    }

    synchronized int listSize() {
        return keywords.size();
    }

    synchronized Vector cloneList() {
        return (Vector) keywords.clone();
    }

    public void setBounds(int x, int y, int w, int h) {
        textbox.setBounds((w - textboxWidth) / 2,
                          0, 
                          textboxWidth, 
                          h);
        editbox.setBounds((w - textboxWidth) / 2,
                          0,
                          textboxWidth,
                          h);
        left.setBounds(0,
                       0,
                       (w - textboxWidth) / 2 - 2,
                       h);
        right.setBounds((w + textboxWidth) / 2 + 2,
                        0,
                        (w - textboxWidth) / 2 - 4,
                        h);
        width = w;
        height = h;

        super.setBounds(x, y, w, h);
    }

    public void mouseEntered(MouseEvent e) {
        Object src = e.getSource();

        if (src == right) {
            right.setHilite(true);
        } else if (src == left) {
            left.setHilite(true);
        }
    }

    public void mouseExited(MouseEvent e) {
        Object src = e.getSource();

        if (src == right) {
            right.setHilite(false);
        } else if (src == left) {
            left.setHilite(false);
        }
    }

    public void mousePressed(MouseEvent e) {
        Object src = e.getSource();

        if (src == right) {
            if (editbox.isShowing()) {
                editbox.setVisible(false);
            }
            textbox.scrollRight();
        } else if (src == left) {
            if (editbox.isShowing()) {
                editbox.setVisible(false);
            }
            textbox.scrollLeft();
        } else if (src == textbox) {
            isDragging = true;
            dragX0 = e.getX(); 
            dragY0 = e.getY();
        }
    }

    public void mouseReleased(MouseEvent e) {
        isDragging = false;
        Object src = e.getSource();

        if (src == right || src == left) {
            textbox.stopScrolling();
        } else if (src == textbox) {
            int x = e.getX();
            int y = e.getY();
            if (x == dragX0 && y == dragY0) {
                editbox.setVisible(true);
                editbox.repaint();
                textbox.setVisible(false);
            }
        }
    }

    public void mouseClicked(MouseEvent e) {}

    public void mouseMoved(MouseEvent e) {}

    public void mouseDragged(MouseEvent e) {
        if (!isDragging) {
            return;
        }
        if (keywords.size() == 0) {
            return;
        }
        Object src = e.getSource();

        if (src == textbox) {
            e.consume();

            int x = e.getX();
            int y = e.getY();
            double rad = Math.sqrt(Math.pow((double) (x - dragX0), 2.0) + 
                                   Math.pow((double) (y - dragY0), 2.0));

            if (rad > mouseStrikeDistance) {
                rad = (double) mouseStrikeDistance;
            }
            textbox.paintStrike(dragX0, dragY0, rad);
            if (rad >= mouseStrikeDistance) {
                isDragging = false;
                strikeKeyword();
            } 
        }
    }

    void spill(String s) {
        if (log != null) {
            log.print(s);
        }
        System.out.println(s);
    }
}

/*
 * class SlowArrowKeywordsListTextbox
 *
 * Canvas for displaying keywords
 */
class SlowArrowKeywordsListTextbox extends Canvas implements Runnable {

    static final int WAITING       = 0;
    static final int SCROLLING     = 1;
    static final int BLINKING      = 2;

    int scrollPos;
    private SlowArrowKeywordsList parent;
    private Dimension d;
    private Image imgTextScroll = null;
    private Image imgHilite = null;
    private Graphics gTextScroll;
    private Graphics gHilite;
    private Font font;
    private int charWidth;
    private int scrollDirection = 0;
    private boolean stopFlag = true;
    private boolean isHilited = false;
    private volatile Thread anim = null;
    private int numKeywords;
    private int state;
    private int scrollInc;
    private int scrollDelay;
    private int blinkDelay;
    private int numBlinks;
    private Color strikeBackColor;
    private int weightH;
    private int pixelsPerWeight;
    private int weightBottomMargin;

    SlowArrowKeywordsListTextbox(SlowArrowKeywordsList parent) {
        this.parent = parent;

        String prop;
        SlowArrowConfig conf = parent.parent.conf;
        
        font = new Font("Courier", Font.PLAIN, parent.fontSize);
        FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(font);
        charWidth = fm.charWidth(' ');
        scrollPos = 0;

        scrollInc = 
                conf.getIntegerProperty("keywordsTextboxScrollInc");
        scrollDelay = 
                conf.getIntegerProperty("keywordsTextboxScrollDelay");
        blinkDelay = 
                conf.getIntegerProperty("keywordsTextboxBlinkDelay");
        numBlinks = 
                conf.getIntegerProperty("keywordsTextboxNumBlinks");
        strikeBackColor = 
                conf.getColorProperty("strikeBackColor");
        weightH = 
                conf.getIntegerProperty("keywordsTextboxWeightHeight");
        pixelsPerWeight = 
                conf.getIntegerProperty("keywordsTextboxPixelsPerWeight");
        weightBottomMargin = 
                conf.getIntegerProperty("keywordsTextboxWeightBottomMargin");
    }

    synchronized void keywordAdded(String keyword) {
        if (imgTextScroll == null) {
            return;
        }

        stopScrolling();

        drawKeywords();

        gHilite.setColor(parent.hiliteColor);
        gHilite.fillRect(0, 0, d.width, d.height);
        gHilite.setColor(parent.backColor);
        int len = keyword.length();
        gHilite.drawString(keyword, 
                           (d.width - len * charWidth) / 2, 
                           d.height - 4);
        repaint();

        // Animate
        state = BLINKING;
        notify();
    }

    synchronized void keywordStriked() {
        try {
            anim.sleep(500);
        } catch (InterruptedException e) {}
        drawKeywords();
        if (scrollPos > (numKeywords - 1) * d.width) {
            scrollPos = (numKeywords - 1) * d.width;
        }
        scrollPos = (scrollPos < 0) ? 0 : scrollPos;
        repaint();
    }

    synchronized void drawKeywords() {
        gTextScroll.setColor(parent.backColor);
        gTextScroll.fillRect(0, 0, parent.max * d.width, d.height);

        Vector v = parent.cloneList();
        numKeywords = v.size();
        for (int i = 0; i < numKeywords; i++) {
            String s = ((SlowArrowKeyword) v.elementAt(i)).keyword;
            int len = s.length();
            int x = i * d.width + (d.width - len * charWidth) / 2;
            gTextScroll.setColor(parent.textColor);
            gTextScroll.drawString(s, x, d.height - 4);
            
            // Draw indicator for keyword weight
            int wt = ((SlowArrowKeyword) v.elementAt(i)).weight;
            int wtW = pixelsPerWeight * wt + 1;
            if (wtW > d.width) {
                wtW = d.width;
            }
            x = i * d.width + (d.width - wtW) / 2;
            gTextScroll.setColor(parent.weightColor);
            for (int xLine = x; xLine < x + wtW; xLine += pixelsPerWeight) {
                gTextScroll.drawLine(xLine,
                                     d.height - weightBottomMargin - weightH,
                                     xLine,
                                     d.height - weightBottomMargin);
            }
            //gTextScroll.fillRect(x, 
            //                     d.height - weightBottomMargin - weightH,
            //                     wtW,
            //                     weightH);
        }
        gTextScroll.setColor(parent.textColor);
        gTextScroll.fillRect(0, 0, 3, 3);
        gTextScroll.fillRect(0, d.height - 3, 3, 3);
        
        int floor1 = (numKeywords > 0) ? numKeywords : 1;
        gTextScroll.fillRect(floor1 * d.width - 3, d.height - 3, 3, 3);
        gTextScroll.fillRect(floor1 * d.width - 3, 0, 3, 3);
    }

    public void start() {
        anim = new Thread(this);
        anim.setPriority(Thread.MAX_PRIORITY);
        state = WAITING;
        anim.start();
    }

    public synchronized void stop() { 
        if (anim != null) {
            Thread moribund = anim;
            anim = null;
            moribund.interrupt(); 
        }
    }

    public void run() {
        Thread currentThread = Thread.currentThread();
        while (currentThread == anim) {
            switch (state) {
            case SCROLLING:
                if (scrollDirection == 1) {
                    scrollPos += scrollInc;
                    repaint();
                    if (scrollPos >= d.width * (numKeywords - 1)) {
                        scrollPos = d.width * (numKeywords - 1);
                        state = WAITING;
                        break;
                    }
                } else if (scrollDirection == -1) {
                    scrollPos -= scrollInc;
                    repaint();
                    if (scrollPos <= 0) {
                        scrollPos = 0;
                        state = WAITING;
                        break;
                    }
                } else {
                    state = WAITING;
                    break;
                }

                if ((scrollPos % d.width) == 0) {
                    if (stopFlag) {
                        state = WAITING;
                        break;
                    } else {
                        try {
                            anim.sleep(3 * scrollDelay);
                        } catch (InterruptedException e) {}
                    }
                }
                try {
                    anim.sleep(scrollDelay);
                } catch (InterruptedException e) {}
                break;

            case BLINKING:
                for (int i = 0; i < numBlinks; i++) {
                    isHilited = true;
                    repaint();
                    try {
                        anim.sleep(blinkDelay);
                    } catch (InterruptedException e) {}

                    isHilited = false;
                    repaint();
                    try {
                        anim.sleep(blinkDelay);
                    } catch (InterruptedException e) {}
                }
                state = WAITING;
                break;

            case WAITING:
                try {
                    synchronized(this) {
                        while (state == WAITING) {
                            wait();
                        }
                    } 
                } catch (InterruptedException e) {}
                break;

            default:
                break;
            }
        }
    }

    private void initGraphics() {
        d = getSize();

        imgTextScroll = createImage(parent.max * d.width, d.height);
        gTextScroll = imgTextScroll.getGraphics();
        gTextScroll.setFont(font);

        imgHilite = createImage(d.width, d.height);
        gHilite = imgHilite.getGraphics();
        gHilite.setFont(font);

        drawKeywords();
        scrollPos = 0;
        start();
    }

    synchronized void setHilite(boolean b) {
        isHilited = b;
        repaint();
    }

    synchronized void scrollRight() {
        stopFlag = false;
        scrollDirection = 1;
        state = SCROLLING;
        notify();
    }

    synchronized void scrollLeft() {
        stopFlag = false;
        scrollDirection = -1;
        state = SCROLLING;
        notify();
    }

    void stopScrolling() {
        stopFlag = true;
    }

    void paintStrike(int x0, int y0, double rad) {
        x0 += scrollPos;
        int r = (int) rad;
        int diam = 2 * r - 1;
        int ovalX = x0 - r + 1;
        int ovalY = y0 - r + 1;
        gTextScroll.setColor(strikeBackColor);
        gTextScroll.fillOval(ovalX, ovalY, diam, diam);
        gTextScroll.setColor(parent.backColor);
        int root2 = (int) (rad / Math.sqrt(2.0));
        int xX1 = x0 - root2;
        int xX2 = x0 + root2;
        int xY1 = y0 - root2;
        int xY2 = y0 + root2;
        gTextScroll.drawLine(xX1, xY1, xX2, xY2);
        gTextScroll.drawLine(xX1 + 1, xY1, xX2, xY2 - 1);
        gTextScroll.drawLine(xX1, xY1 + 1, xX2 - 1, xY2);
        gTextScroll.drawLine(xX1, xY2, xX2, xY1);
        gTextScroll.drawLine(xX1, xY2 - 1, xX2 - 1, xY1);
        gTextScroll.drawLine(xX1 + 1, xY2, xX2, xY1 + 1);
        repaint(ovalX - scrollPos, ovalY, diam, diam);
    }

    public void paint(Graphics g) {
        if (imgTextScroll == null) {
            initGraphics();
        }
        if (isHilited) {
            g.drawImage(imgHilite, 0, 0, this);
        } else {
            g.drawImage(imgTextScroll, -scrollPos, 0, this);
        }
    }

    public void update(Graphics g) {
        paint(g);
    }
}

/*
 * class SlowArrowKeywordsListScroller     
 * 
 * Button for scrolling keywords textbox.
 */
class SlowArrowKeywordsListScroller extends Canvas {

    private SlowArrowKeywordsList parent;
    private Image imgNormal = null;
    private Image imgHilite = null;
    private boolean isHilited = false;

    SlowArrowKeywordsListScroller(SlowArrowKeywordsList parent) {
        this.parent = parent;
    }

    private void initGraphics() {
        Dimension d = getSize();

        imgNormal = createImage(d.width, d.height);
        Graphics g = imgNormal.getGraphics();
        g.setColor(parent.backColor);
        g.fillRect(0, 0, d.width, d.height);
        g.dispose();

        imgHilite = createImage(d.width, d.height);
        g = imgHilite.getGraphics();
        g.setColor(parent.hiliteColor);
        g.fillRect(0, 0, d.width, d.height);
        g.dispose();
    }

    void setHilite(boolean b) {
        isHilited = b;
        repaint();
    }

    public void paint(Graphics g) {
        if (imgNormal == null) {
            initGraphics();
        }
        if (isHilited) {
            g.drawImage(imgHilite, 0, 0, this);
        } else {
            g.drawImage(imgNormal, 0, 0, this);
        }
    }

    public void update(Graphics g) {
        paint(g);
    }
}

/*
 * class SlowArrowKeywordsListEditbox
 *
 * Little textbox for adding new keywords
 */
class SlowArrowKeywordsListEditbox extends Canvas implements KeyListener,
                                                             TimerListener {

    private SlowArrowKeywordsList parent;
    private String text;
    private Dimension d = null;
    private Font font;
    private int charWidth;
    private Timer blinky = null;
    private int blinkDelay;
    private int maxBlinkCount;
    private int blinkCount;
    private boolean isCaretOn;

    SlowArrowKeywordsListEditbox(SlowArrowKeywordsList parent) {
        this.parent = parent;
        setCursor(new Cursor(Cursor.TEXT_CURSOR));
        addKeyListener(this);
        font = new Font("Courier", Font.PLAIN, parent.fontSize);
        FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(font);
        charWidth = fm.charWidth(' ');
        setBackground(Color.black);
        setForeground(Color.lightGray);
        text = "";
        SlowArrowConfig conf = parent.parent.conf;
        blinkDelay = 
                conf.getIntegerProperty("keywordsEditboxBlinkDelay");
        maxBlinkCount = 
                conf.getIntegerProperty("keywordsEditboxIdleTime") / blinkDelay;
    }

    public void setVisible(boolean b) {
        if (b) {
            blinkCount = 0;
            isCaretOn = true;
            if (blinky == null) {
                blinky = new Timer(this, blinkDelay, -1);
            }
            blinky.start();
            parent.parent.currentKeyListener = this;
            requestFocus();
        } else {
            if (blinky != null) {
                blinky.interrupt();
                blinky = null;
            }
            parent.textbox.setVisible(true);
            parent.parent.currentKeyListener = null;
            transferFocus();
        }
        super.setVisible(b);
    }
    
    public void timerTriggered() {
        blinkCount++;
        if (blinkCount < maxBlinkCount) {
            isCaretOn = !isCaretOn;
            repaint();
        } else {
            setVisible(false);
        }
    }
        
    public void paint(Graphics g) {
        //requestFocus();
        if (d == null) {
            d = getSize();
        }

        g.setFont(font);
        g.clearRect(0, 0, d.width, d.height);
        int len = text.length();
        if (len > 0) {
            g.drawString(text, (d.width - len * charWidth) / 2, d.height - 4);
        }
        g.setColor(parent.hiliteColor);
        if (isCaretOn) {
            int caretX = (d.width + len * charWidth) / 2;
            g.drawLine(caretX, 2, caretX, d.height - 2);
        }
    }

    public void update(Graphics g) {
        paint(g);
    }

    public void keyTyped(KeyEvent e) {}

    public void keyReleased(KeyEvent e) {}

    public void keyPressed(KeyEvent e) {
        blinkCount = 0; // reset count
        int code = e.getKeyCode();
        if (code == KeyEvent.VK_ESCAPE || code == KeyEvent.VK_ENTER) {
            setVisible(false);
            if (code == KeyEvent.VK_ENTER) {
                if (Debug.DEBUGGING_ON) {
                    parent.spill("User added keyword " + text);
                }
                parent.addKeyword(text);
            }
            text = "";
        } else if (code == KeyEvent.VK_BACK_SPACE) {
            int len = text.length();
            text = text.substring(0, len - 1);
            repaint();
        } else {
            char c = e.getKeyChar();
            if (Character.isLetterOrDigit(c) || c =='-' || c == '.') {
                text += c;
                repaint();
            }
        }
    }
}
