/*
 * SlowArrowKiller.java    
 * Version 2.2
 * 2001/10/17
 * Written by Victor Liu See-le <victor@n-gon.com>
 * Copyright GPL
 */

package TheSlowArrowofBeauty;

import java.awt.*;
import java.awt.event.*;

/*
 * class SlowArrowKiller 
 * 
 * Shuts down everything.
 */
class SlowArrowKiller extends Slider implements MouseListener {

    protected final static int X_SIZE = 9;

    protected SlowArrow parent;
    protected Image img = null;
    protected int scrollPos;

    SlowArrowKiller(Point origin, 
                    Point slideExtent, 
                    int slideSteps, 
                    int slideDelay,
                    SlowArrow parent) {
        super(origin, slideExtent, slideSteps, slideDelay);
        this.parent = parent;
        addMouseListener(this);
    }

    protected void initGraphics() {
        Dimension d = getSize();

        int tabW = parent.conf.getIntegerProperty("killerTabWidth");
        int fontSize = parent.conf.getIntegerProperty("fontSize");
        Color backC = parent.conf.getColorProperty("killerBackColor");
        Color textC = parent.conf.getColorProperty("killerTextColor");
        Color tabC = parent.conf.getColorProperty("killerTabColor");
        Color xC = parent.conf.getColorProperty("killerXColor");
        Color outlineC = parent.conf.getColorProperty("killerTabOutlineColor");

        img = createImage(d.width, d.height);
        Graphics gc = img.getGraphics();

        int tabX = d.width - tabW;
        gc.setColor(backC);
        gc.fillRect(0, 0, tabX, d.height);

        gc.setColor(tabC);
        gc.fillRect(tabX, 0, tabW, d.height);

        gc.setColor(outlineC);
        gc.fillRect(d.width - 3, d.height - 3, 3, 3);
        gc.fillRect(d.width - 3, 0, 3, 3);

        int xX = tabX + (tabW - X_SIZE - 2) / 2;
        int xY = (d.height - X_SIZE) / 2;
        gc.setColor(xC);
        gc.drawLine(xX + 1, xY, xX + X_SIZE - 1, xY + X_SIZE - 2);
        gc.drawLine(xX + 1, xY + 1, xX + X_SIZE - 2, xY + X_SIZE - 2);
        gc.drawLine(xX, xY + 1, xX + X_SIZE - 2, xY + X_SIZE - 1);
        gc.drawLine(xX, xY + X_SIZE - 2, xX + X_SIZE - 2, xY);
        gc.drawLine(xX + 1, xY + X_SIZE - 2, xX + X_SIZE - 2, xY + 1); 
        gc.drawLine(xX + 1, xY + X_SIZE - 1, xX + X_SIZE - 1, xY + 1); 

        Font font = new Font("Courier", Font.PLAIN, fontSize);
        FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(font);
        int asc = fm.getAscent();
        String s = "close";
        int sWidth = fm.stringWidth(s);
        gc.setFont(font);
        gc.setColor(textC);
        gc.drawString(s, tabX - sWidth - 5, (d.height - asc) / 2 + asc);
    }

    public void paint(Graphics g) {
        if (img == null) {
            initGraphics();
        }
        g.drawImage(img, 0, 0, this);
    }

    public void update(Graphics g) {
        paint(g);
    }

    public void mouseEntered(MouseEvent e) {
        e.consume();
        slideOut();
    }

    public void mouseExited(MouseEvent e) {
        e.consume();
        slideIn();
    }

    public void mousePressed(MouseEvent e) {}

    public void mouseReleased(MouseEvent e) {}

    public void mouseClicked(MouseEvent e) {
        e.consume();
        parent.kill();
    }
}


