/*
 * SlowArrowProgressIndicator.java    
 * Version 2.2
 * 2001/10/17
 * Written by Victor Liu See-le <victor@n-gon.com>
 * Copyright GPL
 */

package TheSlowArrowofBeauty;

import java.awt.*;

/*
 * class SlowArrowProgressIndicator
 *
 * Cute lights to indicate progress, or lack thereof
 */
class SlowArrowProgressIndicator extends Canvas {
    private static final Color COLOR_CHOICES[] = {Color.darkGray,
                                                  Color.pink,
                                                  Color.cyan,
                                                  Color.green,
                                                  Color.lightGray,
                                                  Color.blue,
                                                  Color.orange,
                                                  Color.yellow,
                                                  Color.magenta};
    private static final int NUM_CHOICES = 9;

    SlowArrow parent;
    boolean isDisabled = false;
    private int boxW;
    private int boxH;
    private int colorIndices[];
    private int numBoxes;
    private Image img = null;
    private Graphics gc;
    private int fadePercent = 50;

    SlowArrowProgressIndicator(SlowArrow parent) {
        this.parent = parent;
        boxW = parent.conf.getIntegerProperty("progressIndicatorBoxWidth");
        boxH = parent.conf.getIntegerProperty("progressIndicatorBoxHeight");
        fadePercent = parent.conf.getIntegerProperty(
                "progressIndicatorFadePercent");
    }

    void setColorScheme(int i) {
        Color backColor = 
                parent.conf.getColorProperty("displayBackColor" + i);
        setBackground(backColor);
    }

    public void setEnabled(boolean bool) {
        isDisabled = !bool;
        if (isDisabled) {
            paintFadedBoxes();
        }
    }

    void trigger() {
        if (isDisabled) {
            return;
        }

        if (img == null) {
            return;
        }

        paintBoxes();
        Graphics gHurry = getGraphics();
        paint(gHurry);
        gHurry.dispose();
    }

    private void paintBoxes() {
        for (int i = 0; i < numBoxes; i++) {
            int r = (int) (Math.random() * NUM_CHOICES);
            gc.setColor(COLOR_CHOICES[r]);
            gc.fillRect(i * boxW, 0, boxW, boxH);
            colorIndices[i] = r;
        }
    }

    private void paintFadedBoxes() {
        Color backC = getBackground();
        int rBk = backC.getRed();
        int gBk = backC.getGreen();
        int bBk = backC.getBlue();
        for (int i = 0; i < numBoxes; i++) {
            int r = COLOR_CHOICES[colorIndices[i]].getRed();
            int g = COLOR_CHOICES[colorIndices[i]].getGreen();
            int b = COLOR_CHOICES[colorIndices[i]].getBlue();
            int rFade = rBk + ((r - rBk) * fadePercent) / 100;
            int gFade = gBk + ((g - gBk) * fadePercent) / 100;
            int bFade = bBk + ((b - bBk) * fadePercent) / 100;
            gc.setColor(new Color(rFade, gFade, bFade));
            gc.fillRect(i * boxW, 0, boxW, boxH);
        }
        repaint();
    }

    public void paint(Graphics g) {
        if (img == null) {
            Dimension d = getSize();
            g.clearRect(0, 0, d.width, d.height);
            img = createImage(d.width, d.height);
            gc = img.getGraphics();
            numBoxes = d.width / boxW;
            colorIndices = new int[numBoxes];
            paintBoxes();
        } else {
            g.drawImage(img, 0, 0, this);
        }
    }

    public void update(Graphics g) {
        paint(g);
    }
}
