/*
 * SlowArrowScrollIntro.java    
 * Version 2.2
 * 2001/10/17
 * Written by Victor Liu See-le <victor@n-gon.com>
 * Copyright GPL
 */

package TheSlowArrowofBeauty;

import java.awt.*;

/**
 * class SlowArrowScrollIntro
 *
 * Intro animation
 */
class SlowArrowScrollIntro extends Canvas implements Runnable {

    SlowArrow parent;
    private int nDisplays;
    private volatile Thread thisThread = null;
    private Image img = null;
    private Graphics g;
    private int yScroll = 0;
    private int scrollEnd;
    private int scrollInc;
    private int scrollDelay;
    private Dimension dim;

    SlowArrowScrollIntro(SlowArrow parent) {
        this.parent = parent;
        scrollEnd = parent.conf.getIntegerProperty("displayHeight");
        scrollInc = parent.conf.getIntegerProperty("introScrollInc");
        scrollDelay = parent.conf.getIntegerProperty("introScrollAnimRefresh");
    }

    public synchronized void stop() {
        thisThread = null;
    }
    
    public void start() {
        if (img == null) {
            initGraphics();
        }
        if (thisThread == null)    {
            thisThread = new Thread(this);
        }
        thisThread.start();
    }

    public void run() {
        while (yScroll < scrollEnd) {
            yScroll += scrollInc;
            repaint(0, yScroll - scrollInc, dim.width, yScroll);
            try {
                Thread.sleep(scrollDelay);
            } catch(InterruptedException e) {}
        }
    }

    public void paint(Graphics g) {
        if (img == null) {
            initGraphics();
        }
        g.drawImage(img, 0, -scrollEnd + yScroll, this);
    }

    public void update(Graphics g) {
        paint(g);
    }

    private void initGraphics() {

        // Use configuration file and not the SlowArrowDisplay properties.
        // Want to be able to run intro without the displays existing.
        int nDisplays = parent.conf.getIntegerProperty("numDisplays");
        int dW = parent.conf.getIntegerProperty("displayWidth");
        int dSpc = parent.conf.getIntegerProperty("displaySpacing");

        // Calculate bounds.
        int width = dW * nDisplays + dSpc * (nDisplays - 1);
        int height = scrollEnd;
        dim = new Dimension(width, height);
        resize(dim);

        img = createImage(width, height);
        g = img.getGraphics();
        for (int i = 0; i < nDisplays; i++) {
            g.setColor(parent.d[i].backColor);
            g.fillRect(i * (dW + dSpc), 0, dW, height);
        }
    }

    boolean isDone() {
        return !(yScroll < scrollEnd);
    }
}
